import re
from itertools import islice

from vladiate.exceptions import ValidationException, BadValidatorException


class Validator(object):
    """Generic Validator class"""

    def __init__(self, empty_ok=False):
        self.fail_count = 0
        self.empty_ok = empty_ok

    @property
    def bad(self):
        """Return something containing the "bad" fields"""
        raise NotImplementedError

    def validate(self, field, row):
        """Validate the given field. Also is given the row context"""
        raise NotImplementedError


class CastValidator(Validator):
    """Validates that a field can be cast to a float"""

    def __init__(self, **kwargs):
        super(CastValidator, self).__init__(**kwargs)
        self.invalid_set = set([])

    def validate(self, field, row={}):
        try:
            if field or not self.empty_ok:
                self.cast(field)
        except ValueError as e:
            self.invalid_set.add(field)
            raise ValidationException(e)

    @property
    def bad(self):
        return self.invalid_set


class FloatValidator(CastValidator):
    """Validates that a field can be cast to a float"""

    def __init__(self, **kwargs):
        super(FloatValidator, self).__init__(**kwargs)
        self.cast = float


class IntValidator(CastValidator):
    """Validates that a field can be cast to an int"""

    def __init__(self, **kwargs):
        super(IntValidator, self).__init__(**kwargs)
        self.cast = int


class SetValidator(Validator):
    """Validates that a field is in the given set"""

    def __init__(self, valid_set=[], **kwargs):
        super(SetValidator, self).__init__(**kwargs)
        self.valid_set = set(valid_set)
        self.invalid_set = set([])
        if self.empty_ok:
            self.valid_set.add("")

    def validate(self, field, row={}):
        if field not in self.valid_set:
            self.invalid_set.add(field)
            raise ValidationException(
                "'{}' is not in {}".format(field, _stringify_set(self.valid_set, 100))
            )

    @property
    def bad(self):
        return self.invalid_set


class UniqueValidator(Validator):
    """Validates that a field is unique within the file"""

    def __init__(self, unique_with=[], **kwargs):
        super(UniqueValidator, self).__init__(**kwargs)
        self.unique_values = set([])
        self.duplicates = set([])
        self.unique_with = unique_with
        self.unique_check = False

    def _precheck_unique_with(self, row):
        extra = set(self.unique_with) - set(row.keys())
        if extra:
            raise BadValidatorException(extra)
        self.unique_check = True

    def validate(self, field, row={}):
        if field == "" and self.empty_ok:
            return
        if self.unique_with and not self.unique_check:
            self._precheck_unique_with(row)

        key = tuple([field] + [row[k] for k in self.unique_with])
        if key not in self.unique_values:
            self.unique_values.add(key)
        else:
            self.duplicates.add(key)
            if self.unique_with:
                raise ValidationException(
                    "'{}' is already in the column (unique with: {})".format(
                        field, key[1:]
                    )
                )
            else:
                raise ValidationException("'{}' is already in the column".format(field))

    @property
    def bad(self):
        return self.duplicates


class RegexValidator(Validator):
    """Validates that a field matches a given regex"""

    def __init__(self, pattern=r"di^", full=False, **kwargs):
        super(RegexValidator, self).__init__(**kwargs)
        self.failures = set([])
        if full:
            self.regex = re.compile(r"(?:" + pattern + r")\Z")
        else:
            self.regex = re.compile(pattern)

    def validate(self, field, row={}):
        if not self.regex.match(field) and (field or not self.empty_ok):
            self.failures.add(field)
            raise ValidationException(
                "'{}' does not match pattern /{}/".format(field, self.regex)
            )

    @property
    def bad(self):
        return self.failures


class RangeValidator(Validator):
    def __init__(self, low, high, **kwargs):
        super(RangeValidator, self).__init__(**kwargs)
        self.fail_count = 0
        self.low = low
        self.high = high
        self.outside = set()

    def validate(self, field, row={}):
        if field == "" and self.empty_ok:
            return
        try:
            value = float(field)
            if not self.low <= value <= self.high:
                raise ValueError
        except ValueError:
            self.outside.add(field)
            raise ValidationException(
                "'{}' is not in range {} to {}".format(field, self.low, self.high)
            )

    @property
    def bad(self):
        return self.outside


class EmptyValidator(Validator):
    """Validates that a field is always empty"""

    def __init__(self, **kwargs):
        super(EmptyValidator, self).__init__(**kwargs)
        self.nonempty = set([])

    def validate(self, field, row={}):
        if field != "":
            self.nonempty.add(field)
            raise ValidationException("'{}' is not an empty string".format(field))

    @property
    def bad(self):
        return self.nonempty


class NotEmptyValidator(Validator):
    """Validates that a field is not empty"""

    def __init__(self, **kwargs):
        super(NotEmptyValidator, self).__init__(**kwargs)
        self.fail_count = 0
        self.failed = False

    def validate(self, field, row={}):
        if field == "":
            self.failed = True
            raise ValidationException("Row has empty field in column")

    @property
    def bad(self):
        return self.failed


class Ignore(Validator):
    """Ignore a given field. Never fails"""

    def validate(self, field, row={}):
        pass

    @property
    def bad(self):
        pass


def _stringify_set(a_set, max_len, max_sort_size=8192):
    """Stringify `max_len` elements of `a_set` and count the remainings

    Small sets (len(a_set) <= max_sort_size) are displayed sorted.
    Large sets won't be sorted for performance reasons.
    This may result in an arbitrary ordering in the returned string.
    """
    # Don't convert `a_set` to a list for performance reasons
    text = "{{{}}}".format(
        ", ".join(
            "'{}'".format(value)
            for value in islice(
                sorted(a_set) if len(a_set) <= max_sort_size else a_set, max_len
            )
        )
    )
    if len(a_set) > max_len:
        text += " ({} more suppressed)".format(len(a_set) - max_len)
    return text



from __future__ import division
import csv
from collections import defaultdict
from vladiate.exceptions import ValidationException
from vladiate.validators import EmptyValidator
from vladiate import logs


class Vlad(object):
    def __init__(
        self,
        source,
        validators={},
        default_validator=EmptyValidator,
        delimiter=None,
        ignore_missing_validators=False,
        quiet=False,
    ):
        self.logger = logs.logger
        self.failures = defaultdict(lambda: defaultdict(list))
        self.missing_validators = None
        self.missing_fields = None
        self.source = source
        self.validators = validators or getattr(self, "validators", {})
        self.delimiter = delimiter or getattr(self, "delimiter", ",")
        self.line_count = 0
        self.ignore_missing_validators = ignore_missing_validators
        self.logger.disabled = quiet
        self.invalid_lines = set()

        self.validators.update(
            {
                field: [default_validator()]
                for field, value in self.validators.items()
                if not value
            }
        )

    def _log_debug_failures(self):
        for field_name, field_failure in self.failures.items():
            self.logger.debug('\nFailure on field: "{}":'.format(field_name))
            for i, (row, errors) in enumerate(field_failure.items()):
                self.logger.debug("  {}:{}".format(self.source, row))
                for error in errors:
                    self.logger.debug("    {}".format(error))

    def _log_validator_failures(self):
        for field_name, validators_list in self.validators.items():
            for validator in validators_list:
                if validator.bad:
                    self.logger.error(
                        "  {} failed {} time(s) ({:.1%}) on field: '{}'".format(
                            validator.__class__.__name__,
                            validator.fail_count,
                            validator.fail_count / self.line_count,
                            field_name,
                        )
                    )
                    try:
                        # If self.bad is iterable, it contains the fields which
                        # caused it to fail
                        invalid = list(validator.bad)
                        shown = ["'{}'".format(field) for field in invalid[:99]]
                        hidden = ["'{}'".format(field) for field in invalid[99:]]
                        self.logger.error(
                            "    Invalid fields: [{}]".format(", ".join(shown))
                        )
                        if hidden:
                            self.logger.error(
                                "    ({} more suppressed)".format(len(hidden))
                            )
                    except TypeError:
                        pass

    def _log_missing_validators(self):
        self.logger.error("  Missing validators for:")
        self._log_missing(self.missing_validators)

    def _log_missing_fields(self):
        self.logger.error("  Missing expected fields:")
        self._log_missing(self.missing_fields)

    def _log_missing(self, missing_items):
        self.logger.error(
            "{}".format(
                "\n".join(
                    ["    '{}': [],".format(field) for field in sorted(missing_items)]
                )
            )
        )

    def validate(self):
        self.logger.info(
            "\nValidating {}(source={})".format(self.__class__.__name__, self.source)
        )
        reader = csv.DictReader(self.source.open(), delimiter=self.delimiter)

        if not reader.fieldnames:
            self.logger.info(
                "\033[1;33m" + "Source file has no field names" + "\033[0m"
            )
            return False

        self.missing_validators = set(reader.fieldnames) - set(self.validators)
        if self.missing_validators:
            self.logger.info("\033[1;33m" + "Missing..." + "\033[0m")
            self._log_missing_validators()

            if not self.ignore_missing_validators:
                return False

        self.missing_fields = set(self.validators) - set(reader.fieldnames)
        if self.missing_fields:
            self.logger.info("\033[1;33m" + "Missing..." + "\033[0m")
            self._log_missing_fields()
            return False

        for line, row in enumerate(reader):
            self.line_count += 1
            for field_name, field in row.items():
                if field_name in self.validators:
                    for validator in self.validators[field_name]:
                        try:
                            validator.validate(field, row=row)
                        except ValidationException as e:
                            self.failures[field_name][line].append(e)
                            self.invalid_lines.add(self.line_count)
                            validator.fail_count += 1

        if self.failures:
            self.logger.info("\033[0;31m" + "Failed :(" + "\033[0m")
            self._log_debug_failures()
            self._log_validator_failures()
            return False
        else:
            self.logger.info("\033[0;32m" + "Passed! :)" + "\033[0m")
            return True



import pytest

from vladiate.inputs import LocalFile, String
from vladiate.validators import (
    EmptyValidator,
    FloatValidator,
    NotEmptyValidator,
    SetValidator,
    UniqueValidator,
)
from vladiate.vlad import Vlad


def test_initialize_vlad():
    source = LocalFile("vladiate/examples/vampires.csv")
    validators = {
        "Column A": [UniqueValidator()],
        "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
    }
    assert Vlad(source=source, validators=validators).validate()


def test_initialize_vlad_with_alternative_delimiter():
    source = LocalFile("vladiate/examples/bats.csv")
    validators = {
        "Column A": [UniqueValidator()],
        "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
    }
    delimiter = "|"
    vlad = Vlad(source=source, validators=validators, delimiter=delimiter)
    assert vlad.validate()


def test_initialize_vlad_no_source():
    with pytest.raises(TypeError):
        Vlad().validate()


def test_unused_validator_fails_validation():
    source = LocalFile("vladiate/examples/vampires.csv")
    validators = {
        "Column A": [UniqueValidator()],
        "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
        "Column C": [FloatValidator()],
    }

    assert not Vlad(source=source, validators=validators).validate()


def test_validators_in_class_variable_are_used():
    source = LocalFile("vladiate/examples/vampires.csv")

    class TestVlad(Vlad):
        validators = {
            "Column A": [UniqueValidator()],
            "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
        }

    assert TestVlad(source=source).validate()


def test_missing_validators():
    source = LocalFile("vladiate/examples/vampires.csv")

    class TestVlad(Vlad):
        validators = {"Column A": [UniqueValidator()]}

    assert not TestVlad(source=source).validate()


def test_no_fieldnames():
    source = String("")

    class TestVlad(Vlad):
        validators = {"Foo": [], "Bar": []}

    assert not TestVlad(source=source).validate()


def test_fails_validation():
    source = LocalFile("vladiate/examples/vampires.csv")

    class TestVlad(Vlad):
        validators = {"Column A": [EmptyValidator()], "Column B": [EmptyValidator()]}

    vlad = TestVlad(source=source)
    assert not vlad.validate()
    assert vlad.validators["Column A"][0].fail_count == 3
    assert vlad.validators["Column B"][0].fail_count == 3
    assert vlad.invalid_lines == {1, 2, 3}


def test_gt_99_failures():
    source = String("\n".join(["Foo"] + [str(x) for x in range(100)]))

    class TestVlad(Vlad):
        validators = {"Foo": [EmptyValidator()]}

    assert not TestVlad(source=source).validate()


def test_ignore_missing_validators():
    source = LocalFile("vladiate/examples/vampires.csv")

    class TestVlad(Vlad):
        validators = {"Column A": [UniqueValidator()]}

    vlad = TestVlad(source=source, ignore_missing_validators=True)

    assert vlad.validate()
    assert vlad.missing_validators == {"Column B"}


def test_quiet(caplog):
    source = LocalFile("vladiate/examples/vampires.csv")

    class TestVlad(Vlad):
        validators = {
            "Column A": [UniqueValidator()],
            "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
        }

    vlad = TestVlad(source=source, quiet=True)

    assert vlad.validate()
    assert len(caplog.records) == 0


def test_when_bad_is_non_iterable():
    source = String("Column A,Column B\n,foo")

    class TestVlad(Vlad):
        validators = {
            "Column A": [NotEmptyValidator()],
            "Column B": [NotEmptyValidator()],
        }

    vlad = TestVlad(source=source)

    assert not vlad.validate()
    assert vlad.validators["Column A"][0].fail_count == 1
    assert vlad.validators["Column A"][0].bad
    assert vlad.validators["Column B"][0].fail_count == 0
    assert not vlad.validators["Column B"][0].bad



import pytest
from pretend import stub, call, call_recorder

from vladiate.exceptions import MissingExtraException
from vladiate.inputs import S3File, StringIO, String, VladInput
from vladiate.vlad import Vlad


def mock_boto(result):
    try:
        import builtins
    except ImportError:
        import __builtin__ as builtins
    realimport = builtins.__import__

    def badimport(name, *args, **kwargs):
        if name == "boto":
            return result()
        return realimport(name, *args, **kwargs)

    builtins.__import__ = badimport


@pytest.mark.parametrize(
    "kwargs",
    [
        ({"path": "s3://some.bucket/some/s3/key.csv"}),
        ({"bucket": "some.bucket", "key": "/some/s3/key.csv"}),
    ],
)
def test_s3_input_works(kwargs):
    mock_boto(lambda: stub())
    S3File(**kwargs)


@pytest.mark.parametrize(
    "kwargs",
    [
        ({}),
        ({"path": "s3://some.bucket/some/s3/key.csv", "bucket": "some.bucket"}),
        ({"path": "s3://some.bucket/some/s3/key.csv", "key": "/some/s3/key.csv"}),
        ({"bucket": "some.bucket"}),
        ({"key": "/some/s3/key.csv"}),
    ],
)
def test_s3_input_fails(kwargs):
    mock_boto(lambda: stub())
    with pytest.raises(ValueError):
        S3File(**kwargs)


@pytest.mark.parametrize(
    "kwargs",
    [({"string_input": "ColA,ColB\n,"}), ({"string_io": StringIO("ColA,ColB\n,")})],
)
def test_string_input_works(kwargs):
    source = String(**kwargs)
    validators = {"ColA": [], "ColB": []}
    assert Vlad(source=source, validators=validators).validate()


def test_open_s3file():
    new_key = call_recorder(
        lambda *args, **kwargs: stub(get_contents_as_string=lambda: "contents".encode())
    )

    get_bucket = call_recorder(lambda *args, **kwargs: stub(new_key=new_key))

    mock_boto = stub(connect_s3=lambda: stub(get_bucket=get_bucket))

    s3file = S3File("s3://some.bucket/some/s3/key.csv")
    s3file.boto = mock_boto

    result = s3file.open()

    assert get_bucket.calls == [call("some.bucket")]

    assert new_key.calls == [call("/some/s3/key.csv")]

    assert result.readlines() == [b"contents"]


def test_repr_s3file():
    s3_file = S3File("s3://some.bucket/some/s3/key.csv")
    assert repr(s3_file) == "S3File('s3://some.bucket/some/s3/key.csv')"


def test_base_class_raises():
    with pytest.raises(NotImplementedError):
        VladInput()

    class PartiallyImplemented(VladInput):
        def __init__(self):
            pass

    with pytest.raises(NotImplementedError):
        PartiallyImplemented().open()

    with pytest.raises(NotImplementedError):
        repr(PartiallyImplemented())


def test_s3file_raises_when_no_boto():
    def import_result():
        raise ImportError

    mock_boto(import_result)

    with pytest.raises(MissingExtraException):
        S3File()



class ValidationException(Exception):
    """Thrown when validation fails"""

    pass


class BadValidatorException(Exception):
    """Thrown when a validator is bad"""

    def __init__(self, extra):
        message = "Row does not contain the following unique_with fields: {}"
        super(BadValidatorException, self).__init__(message.format(extra))


class MissingExtraException(Exception):
    """Thrown when an extra dependency is missing"""

    def __init__(self):
        super(MissingExtraException, self).__init__(
            "The `s3` extra is required to use the `S3File` class. Install"
            " it via `pip install vladiate[s3]`."
        )



import os
import sys
import inspect

import pytest
from pretend import stub, call, call_recorder

from vladiate import exits
from vladiate.examples import vladfile
from vladiate.examples.vladfile import YourFirstFailingValidator
from vladiate.inputs import String
from vladiate.main import (
    parse_args,
    is_vlad,
    find_vladfile,
    load_vladfile,
    _vladiate,
    main,
    run,
    _is_package,
)
from vladiate.vlad import Vlad


def test_parse_args():
    options = parse_args()

    assert options.list_commands is False
    assert options.processes == 1
    assert options.show_version is False
    assert options.vladfile == "vladfile"
    assert options.vlads == ["test"]


@pytest.mark.parametrize(
    "tup, expected",
    [
        (("foo", "bar"), False),
        (("YourFirstFailingValidator", YourFirstFailingValidator), True),
    ],
)
def test_is_vlad(tup, expected):
    assert is_vlad(tup) == expected


@pytest.mark.parametrize(
    "name, path",
    [("vladfile", "./vladiate/examples/"), ("./vladiate/examples/vladfile", ".")],
)
def test_find_vladfile(name, path):
    assert find_vladfile(name, path)


@pytest.mark.parametrize(
    "path",
    [
        ("./vladiate/examples/vladfile"),
        (os.path.join(sys.path[0], "vladfile")),
        (os.path.join(sys.path[1], "vladfile")),
    ],
)
def test_load_vladfile(path):
    doc, vlads = load_vladfile(path)
    available = dict(
        inspect.getmembers(
            vladfile, lambda x: inspect.isclass(x) and issubclass(x, Vlad) and x != Vlad
        )
    )

    assert set(vlads.keys()) == set(available.keys())


def test_vladiate(monkeypatch):
    put = call_recorder(lambda *args, **kwargs: stub())
    result_queue = stub(put=put)

    validate_result = stub()

    monkeypatch.setattr("vladiate.main.result_queue", result_queue)

    class TestVlad(Vlad):
        source = String("foo")
        validators = {}

        def validate(self):
            return validate_result

    _vladiate(TestVlad)

    assert put.calls == [call(validate_result)]


def test_run(monkeypatch):
    main_ret = stub()
    main = call_recorder(lambda: main_ret)
    exit = call_recorder(lambda x: stub())
    monkeypatch.setattr("vladiate.main.main", main)
    try:
        monkeypatch.setattr("__builtin__.exit", exit)
    except ImportError:
        monkeypatch.setattr("builtins.exit", exit)

    run("__main__")

    assert main.calls == [call()]

    assert exit.calls == [call(main_ret)]


@pytest.mark.parametrize(
    "get, expected", [(lambda: True, exits.OK), (lambda: False, exits.DATAERR)]
)
def test_main_with_multiprocess(monkeypatch, get, expected):
    monkeypatch.setattr(
        "vladiate.main.parse_args",
        lambda: stub(
            list_commands=False,
            show_version=False,
            vladfile=stub(),
            vlads=["Something"],
            processes=2,
            quiet=False,
        ),
    )
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: stub())
    vlad = call_recorder(lambda *args, **kwargs: stub(validate=lambda: stub()))
    vlad.source = stub()

    monkeypatch.setattr(
        "vladiate.main.load_vladfile",
        lambda *args, **kwargs: (None, {"Something": vlad}),
    )

    Pool = call_recorder(
        lambda *args, **kwargs: stub(map=lambda *args, **kwargs: stub())
    )
    monkeypatch.setattr("vladiate.main.Pool", Pool)

    def empty(calls=[]):
        if calls:
            return True
        calls.append(None)
        return False

    result_queue = stub(get=get, empty=empty)
    monkeypatch.setattr("vladiate.main.result_queue", result_queue)

    assert main() is expected


def test_main_with_vlads_in_args(monkeypatch):
    monkeypatch.setattr(
        "vladiate.main.parse_args",
        lambda: stub(
            list_commands=False,
            show_version=False,
            vladfile=stub(),
            vlads=["Something"],
            processes=1,
            quiet=False,
        ),
    )
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: stub())
    vlad = call_recorder(lambda *args, **kwargs: stub(validate=lambda: stub()))
    vlad.source = stub()

    monkeypatch.setattr(
        "vladiate.main.load_vladfile",
        lambda *args, **kwargs: (None, {"Something": vlad}),
    )
    assert main() is exits.OK


def test_main_no_vlads_in_args(monkeypatch):
    monkeypatch.setattr(
        "vladiate.main.parse_args",
        lambda: stub(
            list_commands=False,
            show_version=False,
            vladfile=stub(),
            vlads=[],
            processes=1,
            quiet=False,
        ),
    )
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: stub())

    vlad = call_recorder(lambda *args, **kwargs: stub(validate=lambda: stub()))
    vlad.source = stub()
    monkeypatch.setattr(
        "vladiate.main.load_vladfile",
        lambda *args, **kwargs: (None, {"Something Else": vlad}),
    )
    assert main() == exits.OK


def test_main_missing_vlads(monkeypatch):
    monkeypatch.setattr(
        "vladiate.main.parse_args",
        lambda: stub(
            list_commands=False,
            show_version=False,
            vladfile=stub(),
            vlads=["Something"],
        ),
    )
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: stub())
    monkeypatch.setattr(
        "vladiate.main.load_vladfile",
        lambda *args, **kwargs: (None, {"Something Else": stub()}),
    )
    assert main() == exits.UNAVAILABLE


def test_main_no_vlads_loaded(monkeypatch):
    monkeypatch.setattr(
        "vladiate.main.parse_args",
        lambda: stub(list_commands=False, show_version=False, vladfile=stub()),
    )
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: stub())
    vlads = []
    monkeypatch.setattr(
        "vladiate.main.load_vladfile", lambda *args, **kwargs: (None, vlads)
    )
    assert main() == exits.NOINPUT


def test_main_list_commands(monkeypatch):
    monkeypatch.setattr(
        "vladiate.main.parse_args",
        lambda: stub(list_commands=True, show_version=False, vladfile=stub()),
    )
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: stub())
    vlads = ["Something"]
    monkeypatch.setattr(
        "vladiate.main.load_vladfile", lambda *args, **kwargs: (None, vlads)
    )
    assert main() == exits.OK


def test_main_show_version(monkeypatch):
    monkeypatch.setattr("vladiate.main.parse_args", lambda: stub(show_version=True))
    assert main() == exits.OK


def test_main_no_vladfile(monkeypatch):
    monkeypatch.setattr("vladiate.main.find_vladfile", lambda *args, **kwargs: None)
    assert main() == exits.NOINPUT


@pytest.mark.parametrize(
    "path, expected", [("foo/bar", False), ("vladiate/examples", True)]
)
def test_is_package(path, expected):
    assert _is_package(path) == expected



from vladiate import Vlad
from vladiate.validators import UniqueValidator, SetValidator
from vladiate.inputs import LocalFile


class YourFirstValidator(Vlad):
    source = LocalFile("vampires.csv")
    validators = {
        "Column A": [UniqueValidator()],
        "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
    }


class YourFirstNonCommaDelimitedValidator(Vlad):
    source = LocalFile("bats.csv")
    validators = {
        "Column A": [UniqueValidator()],
        "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
    }
    delimiter = "|"


class YourFirstFailingValidator(Vlad):
    source = LocalFile("potential_vampires.csv")
    validators = {
        "Column A": [UniqueValidator()],
        "Column B": [SetValidator(["Vampire", "Not A Vampire"])],
    }


class YourFirstEmptyValidator(Vlad):
    source = LocalFile("real_vampires.csv")
    validators = {}


class YourSecondEmptyValidator(Vlad):
    source = LocalFile("real_vampires.csv")
    validators = {"Column A": [], "Column B": [], "Column C": []}



""" A collection of exit codes that work on non-UNIX systems """

import os

OK = getattr(os, "EX_OK", 0)
DATAERR = getattr(os, "EX_DATAERR", 65)
NOINPUT = getattr(os, "EX_NOINPUT", 66)
UNAVAILABLE = getattr(os, "EX_UNAVAILABLE", 69)
